﻿package  
{
	import flash.display.MovieClip;
	import flash.events.Event;
	import flash.events.KeyboardEvent;
	import flash.geom.Point;
	import flash.geom.Rectangle;
	import flash.ui.Keyboard;
	
	public class Player extends MovieClip
	{
		private const TOP_SPEED:Number = 8;
		private const GROUND_FRICTION:Number = 0.9;
		private const RUN_FORCE:Number = 0.8;		
		private const JUMP_FORCE:Number = -7;
		private const AIR_CONTROL:Number = 0.2;
		private const GRAVITY_FORCE:Number = 0.7;
		private const MAX_ITERATIONS = 20;
		
		public var bounds:Rectangle = new Rectangle(0, 0, 23.5, 27.5);
		public var velo:Point = new Point(0,0);
		private var level:MovieClip;
		private var drag:Boolean = false;
		private var onGround:Boolean = false;
		private var canJump:Boolean = true;
		
		private var kRight:int = 0;
		private var kLeft:int = 0;
		private var kJump:int = 0;
			// 0 - not pressed
			// 1 - just pressed
			// 2 - held down
				
		private var anim:int = 0;
			// 0 = Idle		
			// 1 = Running
			// 2 = Jump
		
		public function Player() 
		{			
			if (stage) init();
			else addEventListener(Event.ADDED_TO_STAGE, init);
			
			this.addEventListener(Event.ENTER_FRAME, Update);
			stage.addEventListener(KeyboardEvent.KEY_DOWN, KeyDown);
			stage.addEventListener(KeyboardEvent.KEY_UP, KeyUp);
		}
		
		private function init(e:Event = null):void 
		{
			removeEventListener(Event.ADDED_TO_STAGE, init);			
			level = (parent as MovieClip).Level;
		}
		
		public function KeyDown(e:KeyboardEvent) {
			switch  (e.keyCode) {
				case Keyboard.RIGHT:
					if (kRight == 0) kRight = 1;
					break;
				case Keyboard.LEFT:
					if (kLeft == 0) kLeft = 1;
					break;
				case Keyboard.UP:
				case Keyboard.SPACE:
					if (kJump == 0) kJump = 1;
					break;
			}
		}
		
		public function KeyUp(e:KeyboardEvent) {
			switch  (e.keyCode) {				
				case Keyboard.RIGHT:
					kRight = 0;
					break;
				case Keyboard.LEFT:
					kLeft = 0;
					break;
				case Keyboard.UP:
				case Keyboard.SPACE:
					kJump = 0;
					break;
			}
		}
		
		function Update(e:Event) {			
			
			var c:Point = new Point(x, y);
			c = (parent as MovieClip).localToGlobal(c);
			var old:Point = c.clone();
			
			//--------Gravity, Friction, and Velocity Calculation
			
			if (onGround) {			
				if (kLeft != 0) 
					velo.x -= RUN_FORCE;				
				if (kRight != 0) 
					velo.x += RUN_FORCE;
				if (kJump == 1 && canJump) {
					velo.y = JUMP_FORCE;
					canJump = false;
				}					
				if ((kLeft == 0 && kRight == 0) || (kLeft != 0 && kRight != 0)) {
					velo.x *= GROUND_FRICTION;	
					if (Math.abs(velo.x) < 0.5)
						velo.x = 0; // no pixel by pixel creeping.	
				}
			} else {
				velo.y += GRAVITY_FORCE;		
				
				if (kLeft)
					velo.x -= AIR_CONTROL;
				if (kRight)
					velo.x += AIR_CONTROL;
			}
			
			
			if (Math.abs(velo.x) > TOP_SPEED) 
				velo.x = TOP_SPEED * (velo.x < 0? -1:1);
			if (Math.abs(velo.y) > 10)
				velo.y = 10;
				
			c.x += velo.x;
			c.y += velo.y;
			
			//--------World Collision Handling
			//	the basic algorithm in all of these if statements is... is the pixel to the right of me in a wall?
			// 	well lets make sure we're actually in the wall, and then pull ourselves out of it in .1 increments 
			//  so that we can hug the wall. 
			
			if (velo.x < 0) {
				if (level.hitTestPoint(c.x - (bounds.width / 2 + 1), c.y - bounds.height / 2, true)) {
					c.x = findIntersection(old, c, new Point( - (bounds.width / 2 + 1), - bounds.height / 2)).x;	
					velo.x = 0;
				} else if (level.hitTestPoint(c.x - (bounds.width / 2 + 1), c.y - 2, true)) {
					c.x = findIntersection(old, c, new Point( - (bounds.width / 2 + 1), -2)).x;	
					velo.x = 0;
				}
			}
			
			if (velo.x > 0) {
				if (level.hitTestPoint(c.x + (bounds.width / 2 + 1), c.y - bounds.height / 2, true)) {
					c.x = findIntersection(old, c, new Point( (bounds.width / 2 + 1), - bounds.height / 2)).x;	
					velo.x = 0;
				} else if (level.hitTestPoint(c.x + (bounds.width / 2 + 1), c.y - 2, true)) {
					c.x = findIntersection(old, c, new Point( (bounds.width / 2 + 1), -2)).x;
					velo.x = 0;
				}				
			}
				
			
			if (level.hitTestPoint(c.x, c.y - (bounds.height + 1), true))  // Ceiling collisions
				c.y = findIntersection(old, c, new Point(0, - bounds.height)).y;				
			
			if (level.hitTestPoint(c.x - bounds.width / 2 + 2 , c.y+1, true)
				||  level.hitTestPoint(c.x + bounds.width / 2 -2, c.y+1, true)) {
				if (!onGround) {
					if (level.hitTestPoint(c.x - bounds.width / 2 + 2 , c.y , true))
						c.y = findIntersection( old, c, new Point( - bounds.width / 2 + 2, 0)).y;
					else if (level.hitTestPoint(c.x + bounds.width / 2 -2, c.y, true))
						c.y = findIntersection( old, c, new Point( bounds.width / 2 - 2, 0)).y;	
				}
				velo.y = 0;
				onGround = true;
				canJump = true;
				
			} else {			
				onGround = false;
				canJump = false;	
			}
			
			
			//--------Animation Handling
			
			if (onGround) {
				if (Math.abs(velo.x) > 1) 
					PlayAnim(1);			
				if (Math.abs(velo.x) <= 1) 
					PlayAnim(0);				
			} else
				PlayAnim(2);
							
			if (velo.x > 0) 	 this.scaleX = -1.0;
			else if (velo.x < 0) this.scaleX = 1.0;
			
			//-------Key held detection..
			if (kLeft == 1) kLeft = 2;
			if (kRight == 1) kRight = 2;
			if (kJump == 1) kJump = 2;
			
			c = (parent as MovieClip).globalToLocal(c);
			x = c.x;
			y = c.y;
		}
		
		// Uses a bisection method to find the edge of an obstacle (tests the midpoint, and then goes closer or further
		// based on if it is solid at the midpoint)
		private function findIntersection( start:Point, end:Point, offset:Point ):Point {
			var a:Point = new Point(start.x + offset.x, start.y + offset.y);
			var b:Point = new Point();
			var c:Point = new Point(end.x + offset.x, end.y + offset.y);
			for (var i:int = 0; i < MAX_ITERATIONS; i++) {
				b.x = (a.x + c.x) / 2.0;
				b.y = (a.y + c.y) / 2.0;
				if (level.hitTestPoint(b.x, b.y, true)) {
					c = b.clone(); 	// if it was a collision, the edge must be between a and b.
				} else { 		// otherwise, it must be between b and c
					a = b.clone(); 	
					if ((Math.pow(a.x - b.x, 2) + Math.pow(a.y - b.y, 2)) < Math.pow(0.1, 2)) 
						break;											
				}							
			}
			return new Point(a.x - offset.x, a.y - offset.y);
		}
		
		private function PlayAnim(num:int) {
			if (num == 0 && anim != 0) {
				gotoAndPlay("Idle");
				anim = 0;
			} else if (num == 1 && anim != 1) {
				gotoAndPlay("RunStart");
				anim = 1;
			}
			else if (num == 2 && anim != 2) {
				gotoAndPlay("Jump");
				anim = 2;
			}
		}
	}
	
}